

#### All the input is contained in a separate file

source("input_paper.R");   ## Parameter input



### Utility functions

argmax <- function(v)
{
	mx <- v[1];
	ot <- 1;
	for (i in 1:length(v))
		if ((nw <- v[i]) > mx)
		{
			mx <- nw;
			ot <- i;
		}
	ot;
}







### Data structures

um <- matrix(utmin, ncol=Re, byrow=TRUE); ## Low utilities
uM <- matrix(utMax, ncol=Re, byrow=TRUE); ## High utilities


PI <- matrix(truePI, ncol=Re, byrow=TRUE);## True pi's

al <- matrix( alphas, ncol=Re, byrow=TRUE);## alphas

mixutilities <-
	c(0,0,0,0,
	  0,0,0,1,
	  0,0,1,0,
	  0,0,1,1,
	  0,1,0,0,
	  0,1,0,1,
	  0,1,1,0,
	  0,1,1,1,
	  1,0,0,0,
	  1,0,0,1,
	  1,0,1,0,
	  1,0,1,1,
	  1,1,0,0,
	  1,1,0,1,
	  1,1,1,0,
	  1,1,1,1);


testS <-
      c(0,	0,	0,
	0,	0,	1,
	0,	1,	0,
	1,	0,	0);


### For example 1 use trunc(testS*2)
### For example 2 use trunc(testS*5)

mixtureut <- matrix( mixutilities, ncol=Tr+1, byrow=TRUE);

ones <- matrix( rep( 1, Re), nrow=Re);

alsum <- al %*% ones;

## Note that diag(PI %*% t(um)) is the expected lower utilities of each tretament.

testS <- matrix( testS, ncol=Re, byrow=TRUE);## a test data matrix




trm <- 0:Tr;
ani <- function(maxn)  ## Animation
{
	for (n in 0:maxn)
	{
		PoPI <- (1/3 + (n*PI))/(1 + n); #expected posterior prob with n obs.
		plot( append(trm,trm), append(diag(PoPI %*% t(um)),diag(PoPI %*% t(uM)) ));
	#	Sys.sleep(1);
	}
}



PlotUt <- function(low,high, ml="")  ## Plots  Utilities
{
	plot( c(0,0), append(low[1],high[1]), type="b", axes=FALSE,
		xlim=c(0,length(low)-1), ylim=c(0.95*min(low), 1.05*max(high)),
		xlab="Treatments", ylab="Utilities", main=ml);
	axis( 1, 0:length(low)-1);
	axis( 2, signif(seq( 0.95*min(low), 1.05*max(high), length=5), digits = 5));

	for (t in 1:length(low)-1)
	{
		lines( c(t,t), append(low[t+1],high[t+1]), type="b");
	}

	1;
}

PlotRealUt <- function()
{
	PlotUt( diag(PI %*% t(um)), diag(PI %*% t(uM)), ml="Real Utilities");
}


## Plots two sets of utilities
PlotCompareUt <- function( low1, high1, low2, high2, NDS1, NDS2, notr1=TRUE, notr2=TRUE)
{
	op <- par(mfrow=c(1,3), pty = "s");

	PlotRealUt(); ## Real ut.

	xrng <- c( 0, length(low1)+length(low2)-1);
	yrng <- c( 0.95*min(append(low1,low2)), 1.05*max(append(high1, high2)));

	plot( c(0,0), append(low1[1],high1[1]), type="b", axes=FALSE,
		xlim=xrng, ylim=yrng,
		xlab="Treatments", ylab="Utilities",
		main=paste("n=", n, "Stop"));

	if (notr1)
		axis( 1, 0:(length(low1)-1))
	else
		axis( 1, 0:(length(low1)-1), paste(1:length(low1)));

	axis( 2, signif( seq( yrng[1], yrng[2], length=5), digits=5) );

	for (t in 1:length(low1)-1)
	{
		if (any(t == NDS1))
			lines( c(t,t), append( low1[t+1], high1[t+1]),
				type="b", lwd=3)
		else
			lines( c(t,t), append( low1[t+1], high1[t+1]),
				type="b", lwd=1);

	}

	plot( c(0,0), append(low2[1],high2[1]), type="b", axes=FALSE,
		xlim=xrng, ylim=yrng,
		xlab="Treatments", ylab="Utilities",
		main=paste("n=", n, "Continue"));

	if (notr2)
		axis( 1, 0:(length(low2)-1))
	else
		axis( 1, 0:(length(low2)-1), paste(1:length(low2)));

	axis( 2, signif( seq( yrng[1], yrng[2], length=5), digits=5) );

	print(NDS2); print(length(low2));
	for (t in 1:length(low2))
	{
		if (any(t == NDS2))
			lines( c(t,t)-1, append( low2[t], high2[t]),
				type="b", lwd=3)
		else
			lines( c(t,t)-1, append( low2[t], high2[t]),
				type="b", lwd=1);

	}

	par(op);

	1;
}



mixut <- function(mv)  ## Mixes um and uM accoerding to the vector mv
{
	utout <- NULL;

	for (t in 1:(Tr+1))
	{
		utout <- append( utout, mv[t]*uM[t,] + (1-mv[t])*um[t,] );
	}

	matrix(utout, ncol=Re, byrow=TRUE);
}





##  For discrete distribution generation
source("discrete.R");

## generates from a multinomial distribution
Multinomial <- function( n, probs)
{
	sim <- rdiscrete( n, probs);

	sapply( 1:length(probs), function(x) { length(which(sim == x)); } );
}


###### This function simulates responses from PI
###### n is a vector indicating how many patients in each
###### treatment.
###### example: simS( c(0,30,8,6),  PI)

simS <- function( n, pi, tre)
{
	ot <- matrix( 0, nrow=Tr+1, ncol=Re);

	for (t in 1:(Tr+1))
		ot[t,] <- Multinomial( n[t], pi[t,]);

	ot;

}


### if adding one patient do
onePa <- function( axS, pi, tre)
{
	r <- rdiscrete( 1, pi[tre+1,]);

	axS[tre+1,r] <- axS[tre+1,r] + 1;

	axS;
}






## current number of patients in the trail

n <- 0;

## History

S <- matrix( rep( 0, (Tr+1)*Re), ncol=Re, byrow=TRUE);



############# We start with some basic calculations


## Generate S+(t1,r1) and S+(t1,r1)+(t2,r2)

addS <- function( S, t1=0, r1=0, t2=0, r2=0)
{
	axS <- S;
	if (r1 != 0) axS[ t1+1, r1] <- axS[ t1+1, r1] + 1;
	if (r2 != 0) axS[ t2+1, r2] <- axS[ t2+1, r2] + 1;
	axS;
}

## Predictive probability
## All these functions will be called pr( 2, 2), pr( 2, 2, addS( S, t1=2, r1=2)) etc.

pr <- function( r, t, axS)
{
	(al[t+1,r] + axS[t+1,r])/((al + axS) %*% ones)[t+1];
}


## Current expected utility of treatment t

uStar <- function( t, axS, ut)
{
	sm <- 0.0;
	for (r in 1:Re) sm <- sm + ut[t+1,r]*pr( r, t, axS);
	sm;
}

maxuStar <- function( axS, ut)
{
	ot <- uStar( 0, axS, ut)
	for (t in 1:Tr)
		if ((nw <- uStar( t, axS, ut)) > ot)
			ot <- nw;
	ot;
}



## expected utility of stopping the trial

Us <- function( t, axS, ut)
{
	sm <- 0.0;


	for (tp in 1:Tr)
		for (r in 1:Re)
		{
##			print(sm);
			sm <- sm + axS[tp+1,r]*ut[tp+1,r];
		}

	n <- sum(axS);

	if (n > N) print("Us: WARNING: n > N");

	p*uStar( t, axS, ut) + (1-p)*(1/N)*((N-n)*uStar(t, axS, ut) + sm);

####	(uStar( t, axS, ut) + (sum(orgS)*sm) )/(sum(auxS) + 1);
}

## The maximum across all treatments

maxUs <- function( axS, ut)
{
	ot <- Us( 0, axS, ut)
	for (t in 1:Tr)
		if ((nw <- Us( t, axS, ut)) > ot)
			ot <- nw;
	ot;
}

argmaxUs <- function( axS, ut)
{
	mx <- Us( 0, axS, ut);
	ot <- 0;
	for (t in 1:Tr)
		if ((nw <- Us( t, axS, ut)) > mx)
		{
			mx <- nw;
			ot <- t;
		}
	ot;
}


## a vector with all Us's

calUs <- function( axS, ut)
{
#	ot <- NULL;
#	for (t in 0:Tr)
#		ot <- append( ot, Us( t, axS, ut));
#	ot;

	sapply( 0:Tr, function(t) { Us( t, axS, ut); });
}




###### Here we calculate the non dominated clasess in the 2-step lookahead procedure


## We calculate q2 as a function
q2 <- function( axS, tt1, rr1, tt2, ut)
{
	sum <- 0.0;
	for (rr2 in 1:Re)
		sum <- sum +
	 pr( rr2, tt2, addS( axS, t1=tt1, r1=rr1)) *
		maxUs( addS( axS, t1=tt1, r1=rr1, t2=tt2,r2=rr2), ut);
 	sum;
}

maxq2 <- function( axS, tt1, rr1, ut)
{
	ot <- q2( axS, tt1, rr1, 0, ut);
	for (t in 1:Tr)
		if ((nw <- q2( axS, tt1, rr1, t, ut)) > ot)
			ot <- nw;
	ot;
}

## and also the matrix U is calculated as a function

U <- function( axS, tt1, rr1, ut)
{
	n <- sum(axS);

	if (n <= N-2)
		ot <- max( maxq2( axS, tt1, rr1, ut), maxUs( addS( axS, t1=tt1, r1=rr1), ut));
	if (n == N-1)
		ot <- maxuStar( addS( axS, t1=tt1, r1=rr1), ut);

	ot;
}


## so as u1

u1 <- function( axS, t1, ut)
{
	sum <- 0.0;
	for (r1 in 1:Re)
		sum <- sum + pr( r1, t1, axS)*U( axS, t1, r1, ut);
	sum;
}


## Calculate a vector with all u1's with utility ut

calu1 <- function( axS, ut)
{
	ot <- NULL;
	for (t1 in 1:Tr)
		ot <- append( ot, u1( axS, t1, ut));
	ot;
}



## Calculates an array with all the utilitiies for Us using each ut. function

CalAllUs <- function(axS)
{
	apply( mixtureut, 1, function(mv) { calUs( axS, mixut(mv)); } );
}



## Calculates an array with all the utilitiies for u1 using each ut. function

CalAllu1 <- function(axS)
{

	apply( mixtureut, 1, function(mv) { calu1( axS, mixut(mv)); } );
}




## This function plots the utilities for sropping and continuing
## using um and Um only
PlotUti <- function(axS)
{
	PlotCompareUt( 	calUs( axS, um), calUs( axS, uM),
			calu1( axS, um), calu1( axS, uM), notr2=FALSE);
}

## This function plots the utilities for stopping and continuing
## using um and Um and all the other utilities
PlotAllUti <- function( stp, cnt, stpNDS, cntNDS)
{

	PlotCompareUt( 	stp[,1], stp[,length(stp[1,])], 
				cnt[,1], cnt[,length(cnt[1,])],
				stpNDS, cntNDS,
				notr2=FALSE);


#	for (i in 1:length(stp[1,]))
#		lines( 0:Tr, stp[,i], type="b");
#
#	for (i in 1:length(cnt[1,]))
#		lines( (Tr+1):(2*Tr), cnt[,i], type="b");

											
	1;
}

## Does t1 dominates t2?
dom <- function(t1, t2, uts) { all(uts[t1,] > uts[t2,]); }


## Calculates the non-dominate sets
CalRs <- function( axS, plt=TRUE, wrt=FALSE)
{

### we calculte the non-dom set for continuing

	cnt <- CalAllu1(axS);

	du1 <- rep(0,Tr);

	for (i in 1:(Tr-1))
	  for (j in (i+1):Tr)
	  {
	    if (dom( i, j, cnt)) du1[j] <- 1; ##if i dominates j, remove j
	    if (dom( j, i, cnt)) du1[i] <- 1; ##if j dominates i, remove 1
	  }


### we calculte the non-dom set for stopping in the same way

	stp <- CalAllUs(axS);

	dUs <- rep(0,Tr+1);

	for (i in 1:Tr)
	  for (j in (i+1):(Tr+1))
	  {
	    if (dom( i, j, stp)) dUs[j] <- 1;
	    if (dom( j, i, stp)) dUs[i] <- 1;
	  }



### We make here the plot
	if (plt) PlotAllUti( stp, cnt, which(dUs == 0)-1, which(du1 == 0));

	if (wrt)
	{
		png(filename = paste("simu/simu", n, ".png", sep=""));

		PlotAllUti( stp, cnt, which(dUs == 0)-1, which(du1 == 0));

		dev.off();
	}



		
	list( stopNDS=which(dUs == 0)-1, contNDS=which(du1 == 0), stopU=stp, contU=cnt);

}

adRS <- function(rs)
{
	for (i in 1:length(rs))
		treatments[n+1,rs[i]] <<- 1;
}

Next <- function()
{
	ot <<- CalRs(S); ##Calculates all utilities and non-dominated sets
	cont <- apply( ot$contU, 2, max); ##Max utilities of continuing
	stop <- apply( ot$stopU, 2, max); ##Max utilities of stoping

	if (all(stop >= cont)) ##(for all v¥s)
	{
		print("Stop.  The recommended set is:");
		print(ot$stopNDS);

		FALSE;
	}
	else
	{
		print("Cont.  Randomize among the non-dominated set:");
		print(ot$contNDS);
		adRS(ot$contNDS);

		##randomly select a new treatment from ot$contNDS
		t <- rdiscrete( 1, rep(1,length(ot$contNDS)), ot$contNDS);

		print(paste("New treatment: ", t));
		S <<- onePa( S, PI, t); ##add its response to S
		TRUE;
	}
}

### NN = Horizon, starting at nn
simu <- function( NN, nn=0)
{
	N <<- NN;
	n <<- nn;
	p <<- 1/(1+N);

	## treatmets use for randomization at each stage
	treatments <<- matrix( 0, ncol=Re, nrow=N);

	dev.new(width = 9, height = 3);

	while (Next())
	{
		n <<- n+1;
	       if (n == N)
	       {
			print("Stop, maximum number of patients. The recommended set is:");
			print(ot$stopNDS);
			break;
		}

	       print(paste("Enroll patient ", n, ". Responses:"));
	       print(S);
	}


}

##pdf( "plot.pdf", onefile = FALSE,
##	horizontal=FALSE, pointsize=18, height=6)

plotrun <-function()
{


	x <- which(treatments[,1] == 1);
	y <- rep( 1, length(x));

	plot( x, y, xlim=c(0,N), ylim=c(0,Tr),
		xlab="Patient number",
		ylab="Treatment", axes=FALSE, pch=20);

	axis( 1, trunc((0:5)*(N/5)));
	axis( 2, 0:Tr);

	for (i in 2:Tr)
	{
		x <- which(treatments[,i] == 1);
		y <- rep( i, length(x));
		points(x,y, pch=20);
	}
}






##simu(100);














